@@ -143,6 +143,7 @@ async def render_client_state(cs: ClientState) -> None:
     ) = await render_file(
         cs._to_render_editor_text,
         cs._to_render_file_path,
+        Path(cs._file_path) if cs._file_path else None,
         co_build,
         cs._to_render_is_dirty,
     )
@@ -436,6 +437,24 @@ class RenderManager:
                 print(
                     f"TODO: navigate to error on line {data['line']} of file {data['file_path']}."
                 )
+            elif msg == "browser_navigation":
+                # TODO
+                print(f"TODO: browser navigation to {data['pathname']}.")
+                # Translate the path from a URL to a Path, removing the expected prefix of ``/client/id/``.
+                pathname_raw = urllib.parse.unquote(data["pathname"])
+                expected_pathname_prefix = f"/client/{id_}/"
+                assert pathname_raw.startswith(expected_pathname_prefix)
+                pathname = Path(pathname_raw[len(expected_pathname_prefix):])
+
+                # Update the HTML path
+                cs = self._client_state_dict[id_]
+                cs._file_path = pathname
+
+                # Get the location of the project file. TODO
+                pp = cs._project_path
+                if pp:
+                    # For projects, we need to find the source file that maps to this HTML pathname.
+                    pass
             else:
                 print(f"Error: unknown message {msg} with data '{data}'.")
 
@@ -529,7 +548,7 @@ class RenderManager:
         # NOTE: The ``CodeChat_Server start`` CLI command reads this line, then quits. This means (on Windows at least) that all future ``print`` statements will block, preventing the server from shutting down. Outputing info to the logger avoids this problem. Therefore, **do not include print statements after this point in the code**.
         print("The CodeChat Server is ready.\nCODECHAT_READY", file=sys.stderr)
         # Flush this since extension and test code waits for it before connecting to the server/running the rest of a test.
-        sys.stdout.flush()
+        sys.stderr.flush()
         await asyncio.gather(*[self._worker(i) for i in range(num_workers)])
 
     # Process items in the render queue.
